/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <1gaack@informatik.uni-hamburg.de> wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return Moritz Gaack
 * ----------------------------------------------------------------------------
 */

#include <glib/gprintf.h>
#include "GObjectMyObject.h"

#define MY_OBJECT_GET_PRIVATE(obj)     (G_TYPE_INSTANCE_GET_PRIVATE((obj), MY_TYPE_OBJECT, MyObjectPrivate))


G_DEFINE_TYPE(MyObject, my_object, G_TYPE_OBJECT);


typedef struct _MyObjectPrivate MyObjectPrivate;


struct _MyObjectPrivate
{
    gint value;
};


enum
{
    PROP_0,
    PROP_VALUE
};


static void my_object_finalize(GObject * myObject);


static void my_object_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);


static void my_object_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);


static void my_object_class_init(MyObjectClass * class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    
    gobject_class->finalize = my_object_finalize;
    
    g_type_class_add_private(class, sizeof(MyObjectPrivate));
    
    gobject_class->get_property = my_object_get_property;
    gobject_class->set_property = my_object_set_property;
    
    g_object_class_install_property(gobject_class, PROP_VALUE, g_param_spec_int("val", "val", "val", 0, 10, 0, G_PARAM_READWRITE));
}


static void my_object_init(MyObject * myObject)
{
    MyObjectPrivate *priv = MY_OBJECT_GET_PRIVATE(myObject);
    
    priv->value = 0;
}


static void my_object_finalize(GObject *object)
{
    MyObject *myObject = MY_OBJECT(object);
    MyObjectPrivate *priv = MY_OBJECT_GET_PRIVATE(myObject);
    GObjectClass *parent_class = G_OBJECT_CLASS(my_object_parent_class);
    (*parent_class->finalize)(object);
}


MyObject * my_object_new(int value)
{
    MyObject *myObject;
    
    myObject = MY_OBJECT(g_object_new(MY_TYPE_OBJECT, NULL));
    
    my_object_set_value(myObject, value);
    
    return myObject;
}


int my_object_get_value(MyObject *myObject)
{
    MyObjectPrivate *priv;
    
    g_return_val_if_fail(MY_IS_OBJECT(myObject), -1);
    
    priv = MY_OBJECT_GET_PRIVATE(myObject);
    
    return priv->value;
}


void my_object_set_value(MyObject *myObject, int value)
{
    MyObjectPrivate *priv;

    g_return_if_fail(MY_IS_OBJECT(myObject));
    
    priv = MY_OBJECT_GET_PRIVATE(myObject);
    
    priv->value = value;
}


static void my_object_set_property(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
{
    MyObject *myObject = MY_OBJECT(object);
    
    switch(prop_id)
    {
        case PROP_VALUE:
            my_object_set_value(myObject, g_value_get_int(value));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    }
}


static void my_object_get_property(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
{
    MyObject *myObject = MY_OBJECT(object);
    
    switch(prop_id)
    {
        case PROP_VALUE:
            g_value_set_int(value, my_object_get_value(myObject));
            break;
        default:
            G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
    }
}