
/* Copyright (c) 2013, Jean-David Gadina - www.xs-labs.com
 * All rights reserved.
 *
 * Redistribution and use in source (SGML DocBook) and 'compiled' forms (SGML,
 * HTML, PDF, PostScript, RTF and so forth) with or without modification, are
 * permitted provided that the following conditions are met:
 *
 *  + Redistributions of source code (SGML DocBook) must retain the above
 *    copyright notice, this list of conditions and the following disclaimer
 *    as the first lines of this file unmodified.
 *
 *  + Redistributions in compiled form (transformed to other DTDs, converted
 *    to PDF, PostScript, RTF and other formats) must reproduce the above
 *    copyright notice, this list of conditions and the following disclaimer
 *    in the documentation and/or other materials provided with the
 *    distribution.
 *
 * THIS DOCUMENTATION IS PROVIDED BY THE XS-LABS DOCUMENTATION PROJECT "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL XS-LABS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS DOCUMENTATION, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 */




#ifndef __REFERENCE_COUNTING_IMPL_JDG_H__
#define __REFERENCE_COUNTING_IMPL_JDG_H__

#include <stdio.h>
#include <stdlib.h>


/**
 * Typedefinition: MemoryObject ist ein Objekt um Speicher plus zugehörigen 
 * Referenzzähler zu kapseln.
 *
 * @see alloc(size_t)
 * @see retain(void *)
 * @see release(void *)
 */
typedef struct
{
    unsigned int retainCount;
    void * data;
} MemoryObject;


/**
 * Funktion, die Speicher alloziert.
 *
 * Die Implementierung initialisiert den Referenzzähler mit dem Wert Eins und 
 * alloziert mit Hilfe der calloc-Standardfunktion Speicher.
 *
 * @attention Wenn kein Speicher alloziert werden konnte wird ein NULL-Pointer 
 * zurück gegeben.
 *
 * @param size Speichergröße die alloziert werden soll
 * @return Zeiger auf den allozierten Speicherbereich
 * @see retain(void *)
 * @see release(void *)
 */
void * alloc(size_t size);


/**
 * Funktion, die den Besitzanspruch am Memoryobject-Objekt registriert.
 *
 * Die Implementierung inkrementiert den zum MemoryObject-Objekt gehöhrenden 
 * Referenzzähler um eins. Somit sind Programmierer in der Lage Besitz am Objekt 
 * anzumelden.
 *
 * @param ptr Zeiger auf einen Speicherbereich (Datenblock eines MemoryObject-Objektes)
 * @see alloc(size_t)
 * @see release(void *)
 */
void retain(void * ptr);


/**
 * Funktion, die den Besitzanspruch am Memoryobject-Objekt abmeldet.
 *
 * Die Implementierung dekrementiert den zum MemoryObject-Objekt gehöhrenden
 * Referenzzähler um eins. Wenn der Refferenzzähler auf den Wert Null gesetzt wird, wird 
 * der Süeicher nicht mehr benötigt und das MemoryObject-Objekt wird freigegeben.
 *
 * @param ptr Zeiger auf einen Speicherbereich (Datenblock eines MemoryObject-Objektes)
 * @see alloc(size_t)
 * @see retain(void *)
 */
void release(void * ptr);

#endif
